/********************************************************/
/*  Author   .......................... Matthew Ratcliff*/
/*  Program name ...................... OBJDEFS.H       */
/*  Course ............................ CSc 343, W90    */
/*  Instructor ........................ Sabharwal       */
/*  Office ............................ Rm.33, Blue Bld.*/
/*  Phone ............................. 553-5430        */
/*  Date created  ..................... 26-MAR-90       */
/*  System   .......................... IBM PC/AT Compat*/
/*  Language .......................... Turbo C 2.01    */
/*  Operating system .................. MSC/DOS 3.xx    */
/*  Function .......................... Objects def-    */
/*  initions to be operated on by LAB's 4 & 5           */
/********************************************************/

/*\r* Revision History:

DD-MMM-YY By Description
--------- -- ------------------------------------------------
09-APR-90 MR Final cleanup for LAB5.

*\r*/

#if !defined(MATRIX3D)
#include "matrix3d.h"
#endif

#if !defined(LAB)
#include "lab.h"
#endif

/****************************************
* The OBJECT_TYPE definition for our
* lab CAD package follows.
*
* The structure accomodates all geometric
* types that we will deal with,
* POINT, LINE, and POLYGON.
*
* The structure accomodates the ability to
* Undo a previous function, asynchronously.
* This is done algorithmically as follows:
*
* modify_object( OBJECT_TYPE *object )
*  begin
*   newobject = object;
*   newobject->Undo = &object;
*   if (object->Undo != NULL)
*     free_object(object->Undo);
*     object->Undo = NULL;
*
* Any time an object is modified through
* translation, scaling, or rotation, the
* previous object structure is maintained.
* This is done one level deep.  If an object
* is created, rotated, then translated - we
* can undo to the rotated version, but not
* all the way back to the created version.
*
*/

#define AXIS_POINTS 2   /* Number of points in an axis specification    */
#define LINE_POINTS 2
#define POINT_POINTS 1
#define NO_SIDE_PTS 0

#define POINT_EFFECT_SIZE 2


/****************************************
* When a transformation is carried out,
* it is applied to the VECTOR3D_TYPE
* component.  The screen display positions
* are computed and placed in the integer
* elements VIEW_COORD_TYPE.  Thus, any
* time we wish to redisplay an object,
* simply blast out the VIEW_COORD_TYPE
* components, no computations involved.
* Any time we require some other transform
* we shall use the VECTOR3D_TYPE components
* that retain the homogeneous coordinates:
* [x, y, z, h], in floating point format.
*
* Generally it is a 1-1 mapping from the
* x,y values to the display (orthographic,
* parallel projection).  We must maintain
* the z values when we get into rotation,
* however.  Later we may wish to add
* perspective projection, where z elements
* will be used more actively.  We may wish
* to add the ability for the user to specify
* z-coordinates directly later as well.
* Currently, z is always zero, when it comes
* to user input. MapToScreen() does the
* orthographic projection.  This function
* can be ehnanced to support perspective
* and other projection types.
*/


/****************************************
* The basic objects we will create, edit,
* rotate, translate, scale, and destroy
*/

enum OBJECT_IDENTS { POINT_ID, LINE_ID, POLYGON_ID, SPIN3D_ID };


/****************************************
* The operations we will perform on
* the objects.  Keep track of the previous
* operation, allowing the application
* to keep track of this for user feedback.
*/

enum OPERATION_IDENTS { CREATE_OP,
                        TRANSLATE_OP,
                        ROTATEX_OP,
                        ROTATEY_OP,
                        ROTATEZ_OP,
                        ROTATEXYZ_OP,
                        SCALE_OP,
                        SPIN3D_OP,
                        DELETED_OP,
                        UNDELETED_OP };

/****************************************
* Note that all DELETEs are "soft". This
* allows the application to UNDO a delete.
* In fact the user may be able to review
* a complete DELETE queue.  Objects marked
* as deleted are not drawn, normally.
* However, if the user wishes to review
* the delete list, then the application
* can step through the deleted object list,
* making any item from it visible. The
* user may then select RESTORE, NEXT, DONE;
* for example.  Restore will then mark
* the object as UNDELETED and move it
* back into the active objects list.
*/


typedef struct OBJECT_TYPE
  {
  int   Ident;                  /* OBJECT_TYPE                          */
  int   Operation;              /* OPERATION_TYPE                       */
  int   NumPts;                 /* Total points in object               */
  int   PtsPerSide;             /* Total points per polygon side, if    */
                                /* SPIN3D object                        */
  VIEW_COORD_TYPE sMin;         /* Min.x, Min.y of this object's total  */
                                /* point list                           */
  VIEW_COORD_TYPE sMax;
  VECTOR3D_TYPE   vCenter;      /* Center of object                     */
  VIEW_COORD_TYPE sAxis[2];     /* Axis of rotation, if not NULL, points*/
  VECTOR3D_TYPE   vAxis[2];     /* to two points - arbitrary axis for   */
                                /* this SPIN3D object                   */
  VIEW_COORD_TYPE *sPts;        /* Array of NumPts of vector/display data*/
  VECTOR3D_TYPE   *vPts;        /* Array of NumPts of vector/display data*/
struct OBJECT_TYPE *Undo;       /* If not NULL, then previous object to */
                                /* use if UNDO selected. No computations!*/
  } OBJECT_TYPE;


/****************************************
* OBJECT_TYPE description:
*
* int Ident - Identifier for this object,
* it will be equal to one of the values
* in the OBJECT_IDENTS enumeration
* list above. (Always use the enumerated
* entries, not numeric constants - there
* isn't any guarantee that they will always
* match up! In Ada the compiler enforces
* enumerations as strict data types, which
* prevents messy screw ups!)
*
* int Operation - Operation identification.
* This tells what the LAST OPERATION was
* that created the current instance of
* this object.  If an object is DELETED and
* then UNDELETED, then UNDELETED will be in
* the Operation field.  This field allows
* the application program to report status
* on the last operation performed on any
* object.
*
* int NumPts - This is a count of the total
* number of points in this object.
*
* int PtsPerSide - This is a count of the
* number of points in a side of a SPIN3D
* object operation type.  The point list
* for an object is always in sequential order.
* However, if we spin an object 360 degrees
* at increments of 10 degrees, then we will
* have 36 sets of polygon points. Rather than
* creating 36 separate objects, we simply
* stack them one after the other in the
* OBJECT_TYPE structure, updating NumPts as
* we go.  Before the SPIN3D, however, we copy
* NumPts to PointsPerSide.  This allows the
* application program to draw each side as
* if it were a separate polygon, then step
* through the points connecting vertices.
* It's the most compact way to represent
* such a complicated structure, without
* having to recompute the points all the
* time!
*
* Axis - Screen and vector coordinates
* used to keep the axis of a SPIN3D object.
* Before an object can be
* spun, an axis must be specified.
* The user sets the two end points of the
* Axis and they are stored here.  This
* information is used when generating the
* 3D spin object, and allows us to draw
* a "center-line" axis for the object.
*
* VECTOR3D_TYPE vPts - NumPts collection
* of homogeneous coordinates for the
* object to be displayed.
*
* VIEW_COORD_TYPE sPts - NumPts collection
* of screen coordinates.
*
* OBJECT_TYPE *Undo - If this pointer is
* NULL, then we don't have anything to
* UNDO.  If not null, then, when the
* user selects UNDO and then this OBJECT,
* then the Undo object is made the
* current object, in place of this object.
* The current object then becomes the Undo
* object, and remains so until some other
* OPERATION is carried out on it.  When
* an operation is performed, if the Undo
* pointer is not null, the object is freed
* and the pointer set to null.  Then a
* new object is created and its undo
* pointer is set to the current object.
* It is possible to keep a linked list
* of UNDOs, but this would be very
* wasteful of memory, and the user probably
* wouldn't remember more than a couple
* of previous operations anyway.  One level
* of undo is fine.  Note that Undo's may be
* completely asyncrhronous, with this method.
* The user may select an object, perform an
* operation, then manipulate many other objects,
* come back to the first object, and undo its
* previous operation.
*
*/

